use Test::More;

my $dbname = $ENV{TIME_MOMENTX_BOKUTIN_TEST_DB};
plan skip_all => "\$ENV{TIME_MOMENTX_BOKUTIN_TEST_DB} is not set." unless $dbname;

my $connect_info = {
    dsn            => "dbi:Pg:dbname=$dbname",
    quote_names    => 1,
    on_connect_do  => [
        "set lc_time = 'ja_JP.UTF-8'",
    ],
};
system("dropdb $dbname");
system("createdb $dbname");
system("sqitch deploy db:pg://localhost/$dbname");
#system("script/testdb_schema_loader.pl");

use lib "t/lib";
require TestDB::Schema;
my $schema = TestDB::Schema->connect( $connect_info );
my $rs = $schema->resultset('Album');

my $item = $rs->new_result({});

is $item->now_created_at, undef;
is $item->now_updated_at, undef;
is $item->cur_created_at, undef;
is $item->cur_updated_at, undef;
is $item->ref_created_at, undef;
is $item->ref_updated_at, undef;

$item->insert;

isa_ok $item->now_created_at, "Time::Moment";
isa_ok $item->now_updated_at, "Time::Moment";
is $item->cur_created_at, undef; # default_value は schema deploy のみで、セットされる訳ではない
is $item->cur_updated_at, undef; # 読み出すには discard_changes
isa_ok $item->ref_created_at, "SCALAR"; # discard_changes しないと \"CURRENT_TIMESTAMP" のまま
isa_ok $item->ref_updated_at, "SCALAR";

is $item->now_created_at, $item->now_updated_at; # nanoseconds まで同じにする

###################################################################

# db0 -> timestamp without time zone
# db1 -> timestamp with    time zone
# pl0 -> column => {}
# pl1 -> column => { timezone => "Asia/Tokyo" } # 今のところ感知しない

sub tmx { Time::MomentX::BOKUTIN::tmx(@_) }
{
    $schema->storage->dbh->do( "set timezone = 'UTC'" );
    $ENV{TZ} = 'UTC';
    POSIX::tzset();

    # my $tm = tmx('2018-01-01');
    # is $tm, '2018-01-01T00:00:00Z';
    my $ts = '2018-01-01T00:00:00Z';

    $item->update( { db0_pl0_at => $ts, db0_pl1_at => $ts, db1_pl0_at => $ts, db1_pl1_at => $ts } );
    $item->discard_changes;

    is $item->get_column('db0_pl0_at'),    '2018-01-01 00:00:00';
    is $item->db0_pl0_at,                  '2018-01-01T00:00:00Z';
    #is $item->get_column('db0_pl1_at'),    '2018-01-01 00:00:00';
    #is $item->db0_pl1_at,                  '2018-01-01T00:00:00+09:00';
    is $item->get_column('db1_pl0_at'),    '2018-01-01 00:00:00+00';
    is $item->db1_pl0_at,                  '2018-01-01T00:00:00Z';
    #is $item->get_column('db1_pl1_at'),    '2018-01-01 00:00:00+00';
    #is $item->db1_pl1_at,                  '2018-01-01T09:00:00+09:00';
}

{
    $schema->storage->dbh->do( "set timezone = 'UTC'" );
    $ENV{TZ} = 'Asia/Tokyo';
    POSIX::tzset();

    # my $tm = tmx('2018-01-01');
    # is $tm, '2018-01-01T00:00:00Z';
    my $ts = '2018-01-01T00:00:00Z';

    $item->update( { db0_pl0_at => $ts, db0_pl1_at => $ts, db1_pl0_at => $ts, db1_pl1_at => $ts } );
    $item->discard_changes;

    is $item->get_column('db0_pl0_at'),    '2018-01-01 00:00:00';
    is $item->db0_pl0_at,                  '2018-01-01T00:00:00+09:00';
    #is $item->get_column('db0_pl1_at'),    '2018-01-01 00:00:00';
    #is $item->db0_pl1_at,                  '2018-01-01T00:00:00+09:00';
    is $item->get_column('db1_pl0_at'),    '2018-01-01 00:00:00+00';
    is $item->db1_pl0_at,                  '2018-01-01T00:00:00Z';
    #is $item->get_column('db1_pl1_at'),    '2018-01-01 00:00:00+00';
    #is $item->db1_pl1_at,                  '2018-01-01T09:00:00+09:00';
}

done_testing;
