use Test::More;

use utf8;
use strict;
use warnings;
use feature qw(:5.10);

use_ok "Time::MomentX::BOKUTIN";

sub _wrap {
    my ($code) = @_;
    my $warn;
    local $SIG{__WARN__} = sub { $warn //= ''; $warn .= $_[0] };
    eval { $code->() };
    ($@, $warn);
}

{
    ok !exists $SIG{__DIE__};
    my ($err, $warn) = _wrap(
        sub {
            tmx("1970-01-40T00:00:00Z");
        }
    );
    like $err, qr/1970/, 'Include string in error';
}

{
    my ($err, $warn) = _wrap(
        sub {
            local $SIG{__DIE__} = sub { die "custom error\n" };
            tmx("1970-01-40T00:00:00Z");
        }
    );
    unlike $err, qr/1970/;
    like $err, qr/custom error/;
    like $warn, qr/1970/;
}

{
    my ($err, $warn) = _wrap(
        sub {
            local $SIG{__DIE__}  = sub { die bless \@_, 'MyExceptionObject' }; # Exception::Class etc...
            tmx("1970-01-40T00:00:00Z");
        }
    );
    unlike $err, qr/1970/;
    isa_ok $err, "MyExceptionObject";
    like $warn, qr/1970/;
}

done_testing;
